import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import scg.ac.ni.pmf.newgraph.graphdata.Graph;
import scg.ac.ni.pmf.newgraph.graphdata.GraphData;
import scg.ac.ni.pmf.newgraph.graphdata.Vertex;
import scg.ac.ni.pmf.newgraph.graphdata.VertexData;
import scg.ac.ni.pmf.newgraph.graphdata.EdgeData;
import scg.ac.ni.pmf.newgraph.invariants.InvariantString;
import scg.ac.ni.pmf.newgraph.invariants.VertexIntValues;
  
public class ExerNoK4 implements InvariantString
{
        public String getName()
        {
            return "Exercise 1";
        }
        
        public String getDescription()
        {
            return "Draw a graph in which every vertex belongs to at least " +
                   "two triangles, but some vertex belongs to no K4";
        }

        private void triangles(GraphData data, HashMap tc)
        {
            for (Enumeration i = data.vertices(); i.hasMoreElements(); ) {
                VertexData u = (VertexData) i.nextElement();
                int t = 0;
                for (Enumeration j = data.edges(); j.hasMoreElements(); ) {
                    EdgeData e = (EdgeData) j.nextElement();
                    if (data.isAdjanced(u, (VertexData) e.getVertexA()) &&
                        data.isAdjanced(u, (VertexData) e.getVertexB())) {
                            t++;
                        }
                }
                tc.put(u, new Integer(t));
            }
        }

        public String calculate(GraphData data)
        {
            // calculate the number of triangles in each vertex
/* UNFORTUNATELY, I CANNOT USE THIS, 
   BECAUSE V.BRANKOV DID NOT PUT VertexIntValues PUBLICLY CONSTRUCTABLE!
            TrianglesContaining tc = new TrianglesContaining();
            VertexIntValues vtc;
            tc.calculate(data, vtc);
*/
            HashMap tc = new HashMap();
            triangles(data, tc);

            // how many in less than two triangles?
            int tless = 0;
            VertexData vtless = null;
            for (Enumeration i = data.vertices(); i.hasMoreElements(); ) {
                VertexData u = (VertexData) i.nextElement();
                if (((Integer)tc.get(u)).intValue() < 2) {
                    tless++;
                    vtless = u;
                }
            }
            
            // if the first condition not met, inform
            if (tless==1)
                return "NO - vertex " + vtless.getID() + " in less than two triangles";
            if (tless > 1)
                return "NO - there are several vertices in less than two triangles";
                
            // find the number of K4 to which each vertex belongs
            for (Enumeration i = data.vertices(); i.hasMoreElements(); ) {
                VertexData u = (VertexData) i.nextElement();

                // create a graph of open neighborhood of u 
                Graph uneighborhood = new Graph();
                Map mneighb = new HashMap();

                // its vertices
                for (Enumeration j = u.getNeighbors(); j.hasMoreElements(); ) {
                    VertexData v = (VertexData) j.nextElement();
                    Vertex vv = uneighborhood.createVertex(v.getX(), v.getY());
                    mneighb.put(v, vv);
                }

                // and its edges
                for (Enumeration j = u.getNeighbors(); j.hasMoreElements(); ) {
                    VertexData v = (VertexData) j.nextElement();
                    for (Enumeration k = u.getNeighbors(); k.hasMoreElements(); ) {
                        VertexData w = (VertexData) k.nextElement();
                        if (data.isAdjanced(v, w))
                            uneighborhood.createEdge((Vertex) mneighb.get(v),
                                                     (Vertex) mneighb.get(w));
                    }
                }

                // how many triangles in the neighborhood?
                int ut = 0;
                tc.clear();
                GraphData neighbData = uneighborhood.getData();
                triangles(neighbData,tc);
                for (Enumeration j = neighbData.vertices(); j.hasMoreElements(); ) {
                    VertexData v = (VertexData) j.nextElement();
                    ut += ((Integer)tc.get(v)).intValue();
                }
                
                // exercise solved?
                if (ut==0)
                    return "YES!";
            }
            
            return "NO - each vertex is in some K4.";
        }                
}
