import cern.colt.matrix.*;
import cern.colt.matrix.impl.*;
import cern.colt.matrix.linalg.*;

import scg.ac.ni.pmf.newgraph.graphdata.GraphData;
import scg.ac.ni.pmf.newgraph.graphdata.VertexData;
import scg.ac.ni.pmf.newgraph.invariants.InvariantDoubleMatrix;
import scg.ac.ni.pmf.newgraph.param.Parametrizable;

public class GeneralizedEigenvectors implements InvariantDoubleMatrix, Parametrizable {
        public String getName() {
                return "Eigenvectors, Generalized";
        }

        public String getDescription() {
                return "Eigenvectors of matrix A + beta J.";
        }
        
        public double[][] calculate(GraphData data) {
                int n = data.getOrder();
                int[][] adj = data.adjacencyMatrix();
                double genadj[][] = new double[n][n];
 
                // form a generalized adjacency matrix
                for (int i=0; i<n; i++)
                  for (int j=0; j<n; j++)
                     genadj[i][j]=adj[i][j]+beta;
        
                // find a generalized spectrum
                DoubleMatrix2D cernAdj = new DenseDoubleMatrix2D(genadj);
                EigenvalueDecomposition cernDecomposition = new EigenvalueDecomposition(cernAdj);
                DoubleMatrix2D genV = cernDecomposition.getV();
                return genV.toArray();
        }

        private double beta=0;

        public double getBeta() {
                return beta;
        }

        public void setBeta(double beta) {
                this.beta = beta;
        }

        public String checkParameters() {
                return null;
        }
}