import cern.colt.matrix.*;
import cern.colt.matrix.impl.*;
import cern.colt.matrix.linalg.*;

import scg.ac.ni.pmf.newgraph.graphdata.GraphData;
import scg.ac.ni.pmf.newgraph.graphdata.VertexData;
import scg.ac.ni.pmf.newgraph.invariants.InvariantString;
import scg.ac.ni.pmf.newgraph.param.Parametrizable;

public class MaxBipJustSmallest implements InvariantString {
        public String getName() {
                return "MaxBip Subgraph (smallest eig.)";
        }

        public String getDescription() {
                return "Maximum bipartite subgraph from the eigenspace of the smallest eigenvalue.";
        }
        
        public String calculate(GraphData data) {
                int n = data.getOrder();
                int[][] adj = data.adjacencyMatrix();
                double[][] dbadj = new double[n][n];
                
                // copy to a double[][] matrix
                for (int i=0; i<n; i++)
                    for (int j=0; j<n; j++)
                        dbadj[i][j] = (double)adj[i][j];
 
                // find a spectrum and eigenvectors
                DoubleMatrix2D cernAdj = new DenseDoubleMatrix2D(dbadj);
                EigenvalueDecomposition cernDecomposition = new EigenvalueDecomposition(cernAdj);
                DoubleMatrix1D spectrum = cernDecomposition.getRealEigenvalues();
                DoubleMatrix2D eigenV = cernDecomposition.getV();
                // note that eigenvectors are placed in COLUMNS!
                
                // test only the eigenvectors of the smallest eigenvalue
                double smallestEigenvalue = spectrum.get(0);
                int maxbipsubedges = 0;
                int i=0;
                int[] maxSignVertex = new int[n];
                
                while (spectrum.get(i) < smallestEigenvalue + 0.0001)
                {   // divide the vertices according to the sign of the eigenvector
                    int[] signVertex = new int[n];
                    for (int j=0; j<n; j++)
                        if (eigenV.get(j,i)<0)
                            signVertex[j]=-1;
                        else
                            signVertex[j]=1;
                    // how many edges in such bipartite subgraph?
                    int bipsubedges=0;
                    for (int j=0; j<n; j++)
                        for (int k=j+1; k<n; k++)
                            if ((signVertex[j]*signVertex[k]==-1)
                                 && (adj[j][k]==1))
                                bipsubedges++;
                    // a new record?
                    if (bipsubedges > maxbipsubedges)
                    {
                        maxbipsubedges = bipsubedges;
                        for (int j=0; j<n; j++)
                            maxSignVertex[j] = signVertex[j];
                    }
                    
                    // don't drop into dead loop :)
                    i++;
                }

                // form the result string
                String rezultat = "" + maxbipsubedges + ", part I: ";
                for (int j=0; j<n; j++)
                    if (maxSignVertex[j]==1)
                        rezultat = rezultat + j + ", ";
                rezultat = rezultat + "part II: ";
                for (int j=0; j<n; j++)
                    if (maxSignVertex[j]==-1)
                        rezultat = rezultat + j + ", ";
                return rezultat;
        }
}
